/*******************************************************************************
  NVIC PLIB Implementation

  Company:
    Microchip Technology Inc.

  File Name:
    plib_nvic.c

  Summary:
    NVIC PLIB Source File

  Description:
    None

*******************************************************************************/

/*******************************************************************************
* Copyright (C) 2018 Microchip Technology Inc. and its subsidiaries.
*
* Subject to your compliance with these terms, you may use Microchip software
* and any derivatives exclusively with Microchip products. It is your
* responsibility to comply with third party license terms applicable to your
* use of third party software (including open source software) that may
* accompany Microchip software.
*
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
* EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
* WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
* PARTICULAR PURPOSE.
*
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
* FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
* ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
* THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
*******************************************************************************/

#include "device.h"
#include "plib_nvic.h"


// *****************************************************************************
// *****************************************************************************
// Section: NVIC Implementation
// *****************************************************************************
// *****************************************************************************

void NVIC_Initialize( void )
{
    /* Priority 0 to 7 and no sub-priority. 0 is the highest priority */
    NVIC_SetPriorityGrouping( 0x00 );

    /* Enable NVIC Controller */
    __DMB();
    __enable_irq();

    /* Enable the interrupt sources and configure the priorities as configured
     * from within the "Interrupt Manager" of MHC. */
    NVIC_SetPriority(SVCall_IRQn, 7);
    NVIC_SetPriority(PendSV_IRQn, 7);
    NVIC_SetPriority(SERCOM2_0_IRQn, 7);
    NVIC_EnableIRQ(SERCOM2_0_IRQn);
    NVIC_SetPriority(SERCOM2_1_IRQn, 7);
    NVIC_EnableIRQ(SERCOM2_1_IRQn);
    NVIC_SetPriority(SERCOM2_2_IRQn, 7);
    NVIC_EnableIRQ(SERCOM2_2_IRQn);
    NVIC_SetPriority(SERCOM2_OTHER_IRQn, 7);
    NVIC_EnableIRQ(SERCOM2_OTHER_IRQn);
    NVIC_SetPriority(SERCOM3_0_IRQn, 7);
    NVIC_EnableIRQ(SERCOM3_0_IRQn);
    NVIC_SetPriority(SERCOM3_1_IRQn, 7);
    NVIC_EnableIRQ(SERCOM3_1_IRQn);
    NVIC_SetPriority(SERCOM3_2_IRQn, 7);
    NVIC_EnableIRQ(SERCOM3_2_IRQn);
    NVIC_SetPriority(SERCOM3_OTHER_IRQn, 7);
    NVIC_EnableIRQ(SERCOM3_OTHER_IRQn);
    NVIC_SetPriority(SERCOM7_0_IRQn, 7);
    NVIC_EnableIRQ(SERCOM7_0_IRQn);
    NVIC_SetPriority(SERCOM7_1_IRQn, 7);
    NVIC_EnableIRQ(SERCOM7_1_IRQn);
    NVIC_SetPriority(SERCOM7_2_IRQn, 7);
    NVIC_EnableIRQ(SERCOM7_2_IRQn);
    NVIC_SetPriority(SERCOM7_OTHER_IRQn, 7);
    NVIC_EnableIRQ(SERCOM7_OTHER_IRQn);
    NVIC_SetPriority(USB_OTHER_IRQn, 7);
    NVIC_EnableIRQ(USB_OTHER_IRQn);
    NVIC_SetPriority(USB_SOF_HSOF_IRQn, 7);
    NVIC_EnableIRQ(USB_SOF_HSOF_IRQn);
    NVIC_SetPriority(USB_TRCPT0_IRQn, 7);
    NVIC_EnableIRQ(USB_TRCPT0_IRQn);
    NVIC_SetPriority(USB_TRCPT1_IRQn, 7);
    NVIC_EnableIRQ(USB_TRCPT1_IRQn);
    NVIC_SetPriority(GMAC_IRQn, 7);
    NVIC_EnableIRQ(GMAC_IRQn);
    NVIC_SetPriority(TC0_IRQn, 7);
    NVIC_EnableIRQ(TC0_IRQn);



}

void NVIC_INT_Enable( void )
{
    __DMB();
    __enable_irq();
}

bool NVIC_INT_Disable( void )
{
    bool processorStatus;

    processorStatus = (bool) (__get_PRIMASK() == 0);

    __disable_irq();
    __DMB();

    return processorStatus;
}

void NVIC_INT_Restore( bool state )
{
    if( state == true )
    {
        __DMB();
        __enable_irq();
    }
    else
    {
        __disable_irq();
        __DMB();
    }
}
